#define NO_PIKE_SHORTHAND

#include <apr.h>
#include <apr_portable.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include "global.h"
#include "program.h"
#include "object.h"
#include "mapping.h"
#include "stralloc.h"
#include "interpret.h"
#include "threads.h"
#include "builtin_functions.h"
#include "operators.h"
#include "module_support.h"
#include "pike_types.h"
#include "bignum.h"

/* Avoid warnings about "callback_func" being used as a formal parameter
   name in prototypes in the Subversion includes. */
#define callback_func callback_func_x

#include <svn_pools.h>
#include <svn_error.h>
#include <svn_opt.h>
#include <svn_io.h>
#include <svn_delta.h>
#include <svn_version.h>


extern void svn_pike_check_stream_arg (const char *func, INT32 args, INT32 n,
				       svn_stream_t **ret, apr_pool_t *pool_in,
				       apr_pool_t **pool_out);
extern apr_os_file_t svn_pike_check_file_arg (const char *func, INT32 args, INT32 n);
extern void svn_pike_check_rev_arg (const char *func, INT32 args, INT32 n,
				    svn_opt_revision_t *revision);
extern svn_boolean_t svn_pike_check_recurse_arg (const char *func,
						 INT32 args, INT32 n);
extern svn_boolean_t svn_pike_check_force_arg (const char *func,
					       INT32 args, INT32 n);
extern void svn_pike_set_error (svn_error_t *err);
extern struct pike_string *svn_pike_to_utf8 (struct pike_string *s);
extern void svn_pike_push_utf8 (const char *str);
extern void svn_pike_push_time (apr_time_t t);
extern svn_error_t *svn_pike_make_svn_error (struct svalue *v);
extern void svn_pike_make_propmap (struct svalue *ret, apr_hash_t *props,
				   int utf8_mode);
extern void svn_pike_make_dirents (struct svalue *ret, apr_hash_t *dirents);
extern apr_array_header_t *svn_pike_make_targets_array (struct array *targets,
							apr_pool_t *pool,
							svn_boolean_t allow_empty);
extern apr_hash_t *svn_pike_mapping_to_config_hash (struct mapping *m,
						    apr_pool_t *pool);
extern svn_error_t *svn_pike_log_receiver_func_stub (void *baton,
						     apr_hash_t *changed_paths,
						     svn_revnum_t revision,
						     const char *author,
						     const char *date,
						     const char *message,
						     apr_pool_t *pool);

extern void svn_pike_init_config (void);
extern void svn_pike_init_client (void);
extern void svn_pike_init_delta (void);
extern void svn_pike_init_fs (void);
extern void svn_pike_init_ra (void);
extern void svn_pike_init_auth (void);


extern struct program *svn_pike_delta_editor_program;
extern struct program *svn_pike_auth_provider_program;

struct deltaeditor_obj {
  apr_pool_t *pool;
  const struct svn_delta_editor_t *editor;
  void *editor_baton;
};

extern void svn_pike_delta_editor_from_obj (struct object *obj,
					    const svn_delta_editor_t **editor,
					    void **baton,
					    apr_pool_t *pool);

struct svnstream_obj {
  apr_pool_t *pool;
  svn_stream_t *stream;
};

extern void svn_pike_push_svnstream (svn_stream_t *stream, apr_pool_t *pool);
extern void svn_pike_push_version (const svn_version_t *version);

extern svn_error_t *svn_pike_window_handler_stub (svn_txdelta_window_t *window,
						  void *baton);

#define tRev tName ("Revision", tOr (tInt, tStr))

#define APR_STR0(x) ((char *)STR0(x))

#ifdef WIN32
#define NO_FILE_SPECIFIED INVALID_HANDLE_VALUE
#else
#define NO_FILE_SPECIFIED -1
#endif

